#!/usr/bin/env perl

# If your perl-interpreter is not /usr/bin/perl, then
# set in the first line your perl-interpreter.

$|=1;

use strict;
use Getopt::Long;
use HTTP::Tiny;
use Data::Dumper;

# written by Vinh Hong around 2003, maintained by Detlev Droege (Dr) since then
# $Id: getBibtex.pl 706 2023-06-09 09:15:57Z droege $
# History:
#  2004-01-20 Dr	allow limits for number of refs per call and length
#  2009-04-15 Dr	change DB access for new CMS, keep old method as option
#  2009-07-10 Dr	don't overwrite .bib file if access to server failed
#  2013-03-18 Dr	new URL
#  2018-09-05 Dr	remove '%%' from first output line, biber fails on it (drops first record)
#  2019-09-12 Dr	new URL
#  2020-03-15 Dr	fix postprocessing. no more <pre> in output, so don't drop chars
#  2022-04-21 Dr	more postprocessing. remove empty entries, produced by LitDB for unknown keys
#  2022-09-20 Dr	better error message on access failure
#  2022-11-13 Dr	replace problematic LWP::UserAgent with HTTP::Tiny
#  2023-03-28 Dr	adapt for moved (AGAS internal) LitDB server, ditch oldCMS code

# configuration section: =====================================================

  # my $uklURL = "https://www.uni-koblenz-landau.de/de/koblenz/publikationen/bib.php";
  my $ukAgasURL = "https://aslit.uni-koblenz.de/litdb/bib.php";
  my $Response;
  my $URLkey = "litkeys";
  my $proxy = "";
  my $bibtexFile = "myref.bib";
  my $maxRefsPerCall = 32;	# set to 1 for 1 call per ref
  my $usePost = 0;		# will use GET if 0, POST if 1 (POST works for $optOldCMS only)
  my $useLWP_UA = 0;		# will use LWP::UserAgent if 1, HTTP::Tiny otherwise
  my $Verbose = 0;

# end configuration section ==================================================

&Main();

# *********************************************************************
# * Unterroutinen                                                     *
# *********************************************************************

sub Main
{
  my ($optProxy);
  my ($optBibtexFile);
  my ($optHelp);
  my ($optNumRefs);
  my ($optVerbose);

  GetOptions("proxy=s" => \$optProxy,
     "bibfile=s" => \$optBibtexFile,
     "numrefs=i" => \$optNumRefs,
     "verbose" => \$optVerbose,
     "help" => \$optHelp);

  if (($optHelp) || !(@ARGV))
  {
    die "$0 [--proxy=PROXYURL] [--bibfile=BIBFILE] [--numrefs=NUM] [--help] [--verbose] <bib-keys>\n";
  }

  if ($optVerbose)
  {
    $Verbose = $optVerbose;
  }

  if ($optProxy)
  {
    $proxy = $optProxy;
  }
  print "Proxy: $proxy\n" if ($Verbose);

  if ($optBibtexFile)
  {
    $bibtexFile = $optBibtexFile;
  }
  print "bibtexFile: $bibtexFile\n" if ($Verbose);
  if ($optNumRefs)
  {
    $maxRefsPerCall = $optNumRefs;
  }
  $maxRefsPerCall = 1 if ($maxRefsPerCall < 1);
  print "maxRefsPerCall: $maxRefsPerCall\n" if ($Verbose);

  open (BIB, ">$bibtexFile.tmp") || die "can't create BIB file '$bibtexFile'\n";
  if ($optNumRefs || ! $usePost) {# deal with limited URL length for GET method
    while (@ARGV > $maxRefsPerCall) {
      my @part = splice (@ARGV, 0, $maxRefsPerCall);
      fetchRefsTiny (\*BIB, @part);
    }
  }
  fetchRefsTiny (\*BIB, @ARGV);	# fetch rest (or all for POST method)
  close (BIB);
  if ( (-s "$bibtexFile.tmp") > 0) {
    unlink ($bibtexFile);
    rename ("$bibtexFile.tmp", $bibtexFile);
  }
  else {
    unlink ("$bibtexFile.tmp");
    print STDERR "Could not access LitDB server, keeping old .bib file\n";
  }

  print "Written into: $bibtexFile\n" if ($Verbose);
}

sub fetchRefsTiny { # uses HTTP::Tiny, no POST
  my $file = shift;
  my @args = @_;

  my ($Request, $Response, $http, $URL);
  my ($i, $quotedRefs, $queryArg);

  return unless (@args);

  $quotedRefs = join (",", @args);			# (a b) -> a,b
  $queryArg = "$URLkey=" . $quotedRefs;
  print "Refs: $quotedRefs\n" if ($Verbose);
  
  my %attributes = { "timeout" => 30, };
  $http = HTTP::Tiny->new( %attributes );

  die "POST access not supported in fetchRefsTiny()" if ($usePost);
  my @urllist = ($ukAgasURL);
  foreach $URL (@urllist) {
    my $getURL = $URL . "?$queryArg";
    print "GET: $getURL\n" if ($Verbose);
    $Response = $http->get($getURL);
    last if ($Response->{success});
  }

  if ($Response->{success}) {
      my $Resp = $Response->{content};
      $Resp =~ s:^<pre>::; # 2009-2019 response was enclosed by <pre>...</pre>
      $Resp =~ s:</pre>$::;
      $Resp =~ s:\@\{\s*,?\s*\}::mg; # remove empty entries, produced by LitDB for unknown keys
      print $file "", $Resp, "\n";
  }
  else {
    printf STDERR "Access failed (%d): %s\n", $Response->{status}, $Response->{reason};
    my $d = Data::Dumper->new([$Response], [qw(Response)]);
    print "Response: ", $d->Dump;
  }
}

1;
